/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2007 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Class
    quadraticFitSnGradData

Description
    Data for the quadratic fit correction snGrad scheme

SourceFiles
    quadraticFitSnGradData.C

\*---------------------------------------------------------------------------*/

#ifndef quadraticFitSnGradData_H
#define quadraticFitSnGradData_H

#include "MeshObject.H"
#include "fvMesh.H"
#include "surfaceFieldsFwd.H"
#include "Matrix.H"
#include "labelList.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                    Class quadraticFitSnGradData Declaration
\*---------------------------------------------------------------------------*/

class quadraticFitSnGradData
:
    public MeshObject<fvMesh, quadraticFitSnGradData>
{
    // Private data
    
        const scalar centralWeight_;
        const label dim_;

        //- minimum stencil size
        const label minSize_;
        
        //- the number of expansions of the stencil
        static const scalar nExpands;

        //- Cells creating stencil for each cell
        labelListList stencilCells_, stencilBounds_;
        
        //- For each cell in the mesh store the values which multiply the
        //  values of the stencil to obtain the gradient for each direction
        List<scalarList> fit_;


    // Private member functions

        label calcFit(const label faci);


public:

    TypeName("quadraticFitSnGradData");


    // Constructors

        explicit quadraticFitSnGradData
        (
            const fvMesh& mesh, 
            const scalar cWeight
        );


    // Destructor

        virtual ~quadraticFitSnGradData() {};


    // Member functions

        //- Return reference to the stencil cells
        const labelListList& stencilCells() const { return stencilCells_; }

        //- Return reference to the stencil boundary faces
        const labelListList& stencilBounds() const { return stencilBounds_; }

        //- Return reference to fit coefficients
        const List<scalarList>& fit() const { return fit_; }

        //- Delete the data when the mesh moves not implemented
        virtual bool movePoints();
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
